
# q-expansion of newform 750.2.a.d, downloaded from the LMFDB on 17 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 750
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [1, 0], [0, 0], [4, -1], [-1, -3], [5, -1], [-2, 6], [-4, 5], [5, -5], [-4, 6], [0, -4], [0, -1], [2, -1], [-2, 6], [7, 1], [2, -1], [-6, -3], [2, 6], [0, 2], [2, -4], [-8, 0], [6, 4], [2, 0], [1, 9], [2, -10], [-4, -6], [11, -5], [-2, 0], [8, -6], [-6, 6], [-12, 4], [-9, -7], [-8, 8], [-1, -1], [10, -12], [-10, 6], [6, -8], [-24, 0], [-10, 5], [6, -1], [-14, 11], [-14, -4], [-4, 0], [0, 6], [-2, -8], [-8, 2], [-8, -9], [-16, -8], [14, -2], [-26, 2], [4, -4], [-20, 8], [7, 5], [-24, 0], [24, -4], [14, 7], [10, -12], [10, -12], [-13, 25], [-3, 17], [-8, 10], [-3, -11], [10, -10], [-4, 8], [10, -6], [19, 3], [0, 4], [18, -24], [2, -10], [-20, 0], [20, -18], [2, -12], [-4, 12], [-8, -7], [-11, 29], [6, 13], [14, -30], [12, 7], [10, -7], [0, -3], [-8, 16], [4, -18], [14, 2], [6, 20], [-10, -14], [16, -18], [2, 9], [8, 16], [-16, 22], [-28, 8], [18, 2], [20, -18], [5, -11], [-10, 13], [-7, 1], [-35, 3], [22, -6], [-7, 5], [0, 20], [6, -12], [-4, 2], [-27, 25], [-6, 8], [-19, 29], [-20, 15], [6, 2], [22, 2], [-14, 28], [-10, 2], [25, -9], [24, -21], [-1, -19], [12, -4], [-7, -3], [24, 6], [-34, 3], [-28, 8], [10, 3], [17, -3], [-13, 17], [-30, 18], [-16, 4], [6, -19], [-4, -16], [-8, -16], [-8, -6], [-4, 14], [38, -16], [-24, 7], [-29, 29], [12, 7], [-43, 3], [38, -26], [23, -7], [34, 13], [23, -7], [-14, -8], [-30, 12], [26, -37], [35, 7], [21, -15], [-4, -16], [12, -29], [22, -10], [38, -20], [46, -14], [24, 15], [-6, 6], [20, -13], [-10, 13], [25, -21], [34, -31], [12, -12], [-6, -13], [6, -14], [-42, 8], [40, -2], [-31, 45], [-16, 14], [-2, -10], [-12, 16], [26, -30], [15, 27], [-31, 15], [-36, 28], [15, -23], [42, -16], [-4, 5]]
